import os
import errno
import configparser
import logging
import shutil
import filecmp 
from platformdirs import *


logging.basicConfig(format='%(levelname)s:%(filename)s:%(message)s',
                    level=logging.INFO)


# ------------------------- Reading *.conf configuration files -----------
class generalconfig:
    def __init__(self, filename, verbose=False):
        if not filename:
            filename = './conf/general.conf'
        if not os.path.exists(filename):
            try:
                from xdg.BaseDirectory import xdg_config_home

                config = xdg_config_home
            except ImportError:
                config = os.path.join(os.getenv('HOME'), '.config')
            if not os.path.exists(config):
                config = '/etc'
            config = os.path.join(config, 'droidlysis')
            if os.path.exists(config):
                filename = os.path.join(config, 'general.conf')
        logging.debug(f'Reading config from {filename}')

        if not os.path.exists(filename):
            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), filename)
        self.config = configparser.ConfigParser()
        self.config.read(filename)

        # get config
        self.APKTOOL_JAR = os.path.expanduser(self.config['tools']['apktool'])
        self.BAKSMALI_JAR = os.path.expanduser(self.config['tools']['baksmali'])
        self.DEX2JAR_CMD = os.path.expanduser(self.config['tools']['dex2jar'])
        self.KEYTOOL = os.path.expanduser(self.config['tools']['keytool'])
        self.SMALI_CONFIGFILE = os.path.join(os.path.dirname(filename),
                                             self.config['general']['smali_config'])
        self.WIDE_CONFIGFILE = os.path.join(os.path.dirname(filename),
                                            self.config['general']['wide_config'])
        self.ARM_CONFIGFILE = os.path.join(os.path.dirname(filename),
                                           self.config['general']['arm_config'])
        self.DISTRIB_KIT_CONFIGFILE = os.path.join(os.path.dirname(filename),
                                                   self.config['general']['kit_config'])

        # duplicate kit configuration for edition
        cache_dir = user_cache_dir('droidlysis', 'cryptax')
        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)
        self.KIT_CONFIGFILE = os.path.join(cache_dir,
                                           self.config['general']['kit_config'])
        if not os.path.exists(self.KIT_CONFIGFILE):
            logging.debug(f'Copying {self.DISTRIB_KIT_CONFIGFILE}'
                          f' to {self.KIT_CONFIGFILE}')
            shutil.copyfile(self.DISTRIB_KIT_CONFIGFILE, self.KIT_CONFIGFILE)
        elif not filecmp.cmp(self.KIT_CONFIGFILE, self.DISTRIB_KIT_CONFIGFILE):
            logging.warning(f'Cached kit configuration {self.KIT_CONFIGFILE} != general distrib config {self.DISTRIB_KIT_CONFIGFILE}. Using cached config. Check this is ok')

        self.SQLALCHEMY = f'sqlite:///{self.config["general"]["db_file"]}'

        # check files are accessible
        for f in [self.APKTOOL_JAR, self.BAKSMALI_JAR,
                  self.DEX2JAR_CMD, 
                  self.SMALI_CONFIGFILE, self.WIDE_CONFIGFILE,
                  self.ARM_CONFIGFILE, self.KIT_CONFIGFILE]:
            if not os.access(f, os.R_OK):
                logging.warning(f'Cannot access {f} - check your configuration file {filename}')

        if not os.access(self.KEYTOOL, os.X_OK):
            logging.warning(f'Cannot access keytool at {self.KEYTOOL} - check your configuration file {filename}')


class droidconfig:
    def __init__(self, filename, verbose=False):
        assert filename is not None, "Filename is invalid"
        assert os.access(filename, os.R_OK) is not False, "File {0} is not readable".format(filename)

        self.filename = filename
        self.configparser = configparser.RawConfigParser()
        if verbose:
            logging.getLogger().setLevel(logging.DEBUG)
        logging.debug("Reading configuration file: '%s'" % (filename))
        self.configparser.read(filename)

    def get_sections(self):
        return self.configparser.sections()

    def get_pattern(self, section):
        return self.configparser.get(section, 'pattern')

    def get_description(self, section):
        try:
            return self.configparser.get(section, 'description')
        except (configparser.NoSectionError, configparser.NoOptionError):
            pass
        return None

    def is_pattern_present(self, pattern):
        for section in self.get_sections():
            section_patterns = self.get_pattern(section).split('|')
            if pattern in section_patterns:
                return True
            for p in section_patterns:
                if p in pattern:
                    # our pattern is more generic
                    return True
        return False

    def get_all_regexp(self):
        # reads the config file and returns a list
        # of all patterns for all sections
        # the patterns are concatenated with a |
        # throws NoSectionError, NoOptionError
        allpatterns = ''
        for section in self.configparser.sections():
            if allpatterns == '':
                allpatterns = self.configparser.get(section, 'pattern')
            else:
                allpatterns = self.configparser.get(section, 'pattern') + '|' + allpatterns
        return bytes(allpatterns, 'utf-8')

    def match_properties(self, match, properties):
        """
        Call this when the recursive search has been done to analyze the results
        and understand which properties have been spotted.

        @param match: returned by droidutil.recursive_search. This is a dictionary
        of matching lines ordered by matching keyword (pattern)

        @param properties: dictionary of properties where the key is the property name
        and the value will be False/True if set or not

        throws NoSessionError, NoOptionError
        """
        for section in self.configparser.sections():
            pattern_list = self.configparser.get(section, 'pattern').split('|')
            properties[section] = False
            for pattern in pattern_list:
                # beware when pattern has blah\$binz, the matching key is blah$binz
                if match[pattern.replace('\\', '')]:
                    logging.debug("Setting properties[%s] = True (matches %s)" % (section, pattern))
                    properties[section] = True
                    break

                



import os
import errno
import re
import shutil
import magic
import hashlib
from collections import defaultdict

"""Those are my own utilities for sample analysis"""


def mkdir_if_necessary(path):
    """
    Creates the directory if it does not exist yet.
    If it exists, does not do anything.
    If path is None (not filled), does not do anything.
    """

    if path is not None:
        try:
            os.makedirs(path)
        except OSError as exc:  # Python >2.5
            if exc.errno == errno.EEXIST and os.path.isdir(path):
                pass
            else:
                raise


def on_rm_tree_error(fn, path, exc_info):
    """
    Error handler for ``shutil.rmtree``.

    rmtree fails in particular if the file to delete is read-only.
    to remove, we attempt to set all permissions and then retry.

    Usage : ``shutil.rmtree(path, onerror=onerror)``
    """
    if fn is os.rmdir:
        os.chmod(path, 777)
        os.rmdir(path)
    elif fn is os.remove:
        os.chmod(path, 777)
        os.remove(path)


def move_dir(src, dst):
    # Move src directory to dst - works even if dst already exists.
    assert os.path.isdir(src), "src must be an existing directory"
    os.system("mv" + " " + src + "/* " + dst)
    shutil.rmtree(src, onerror=on_rm_tree_error)


def sanitize_filename(filename):
    """Sanitizes a filename so that we can create the output analysis directory without any problem.
    We need to consider we might have filenames with Russian or Chinese characters. 
    
    filename is only the 'basename' not an absolute path

    Returns the sanitized name."""
    # we remove any character which is not letters, numbers, _ or .
    return re.sub('[^a-zA-Z0-9_\.]', '', filename)


def listAll(dirName):
    filelist1 = []
    files = os.listdir(dirName)
    for f in files:
        if os.path.isfile(os.path.join(dirName, f)):
            filelist1.append(os.path.join(dirName, f))
        else:
            newlist = listAll(os.path.join(dirName, f))
            filelist1.extend(newlist)           
    return filelist1 


def count_filedirs(dirname):
    """Counts the number of directories and files in a given directory. Counts recursively.
    dirname must be readable.
    Returns:
    nb of directories
    nb of files
    
    This is somewhat the equivalent of: find ./smali -type d -print 
    or -type f
    """
    assert os.access(dirname, os.R_OK), "Can't access directory: "+dirname
    
    dirs = [name for name in os.listdir(dirname) if os.path.isdir(os.path.join(dirname, name))]
    nb_dirs = len(dirs)
    nb_files = len([name for name in os.listdir(dirname) if os.path.isfile(os.path.join(dirname, name))])
    for element in dirs:
        try:
            element_dirs, element_files = count_filedirs(os.path.join(dirname, element))
        except RuntimeError:
            # occurs when too many recursive dir
            element_dirs = 0
            element_files = 0
        nb_dirs += element_dirs
        nb_files += element_files

    return nb_dirs, nb_files


def sha256sum(input_file_name):
    """Computes the SHA256 hash of a binary file
    Returns the digest string or '' if an error occurred reading the file"""
    chunk_size = 1048576  # 1 MB
    file_sha256 = hashlib.sha256()
    try:
        with open(input_file_name, "rb") as f:
            byte = f.read(chunk_size)
            while byte:
                file_sha256.update(byte)
                byte = f.read(chunk_size)
    except IOError:
        print('sha256sum: cannot open file: %s' % input_file_name)
        return ''
    return file_sha256.hexdigest()


def sha1sum(input_file_name):
    """
    Computes the SHA1 hash of a binary file
    Returns the digest string or '' if an error occurred reading the file
    """
    chunk_size = 1048576  # 1 MB
    file_sha1 = hashlib.sha1()
    try:
        with open(input_file_name, "rb") as f:
            byte = f.read(chunk_size)
            while byte:
                file_sha1.update(byte)
                byte = f.read(chunk_size)
    except IOError:
        print('sha1sum: cannot open file: %s' % input_file_name)
        return ''
    return file_sha1.hexdigest()


# -------------------------- File Constants -------------------------
"""Something else than the other file types. We do not support this file type."""
UNKNOWN = 0

"""An APK. It is not possible to differentiate a ZIP from an APK until we have looked inside the ZIP."""
APK = 1

"""A Dalvik Executable file. We do not check the file is valid/accepted by the verifier."""
DEX = 2

"""An ARM ELF executable."""
ARM = 3

"""A Java .class file"""
CLASS = 4

"""A Zip file. Actually, this can also be a JAR or an APK until we have thoroughly checked."""
ZIP = 5

"""A RARed file."""
RAR = 6

"""We can probably add some more later: TAR, TGZ, BZ2..."""


def str_filetype(filetype):
    """Provide as input a droidutil filetype (APK, DEX, ARM...) and returns the corresponding string"""
    if filetype == APK:
        return "APK"
    if filetype == DEX:
        return "DEX"
    if filetype == ARM:
        return "ARM"
    if filetype == CLASS:
        return "CLASS"
    if filetype == ZIP:
        return "ZIP"
    if filetype == RAR:
        return "RAR"
    return "UNKNOWN"
    

def get_filetype(filename):
    """Returns an enumerate for the filetype corresponding to the given absolute filename.
    This function does not open the file or unzip it.
    It will return one of these:
    droidutil.ZIP
    droidutil.RAR
    droidutil.ARM
    droidutil.CLASS
    droidutil.DEX
    droidutil.UNKNOWN
    """
    filetype = magic.from_file(filename)
    if filetype is None:
        # this happens if magic is unable to find file type
        return UNKNOWN
    match = re.search('Zip archive data|zip|RAR archive data|executable, ARM|'
                      'shared object, ARM|Java class|Dalvik dex|Java archive|Android package', filetype)
    if match is None:
        mytype = UNKNOWN
    else:
        typecase = {'Zip archive data': ZIP,
                    'zip': ZIP,
                    'Java archive': ZIP,
                    'RAR archive data': RAR,
                    'executable, ARM': ARM,
                    'shared object, ARM': ARM,
                    'Java class': CLASS,
                    'Dalvik dex': DEX,
                    'Android package': ZIP,  # droidsample needs ZIP type to do all the processing
                    'None': UNKNOWN
                    }
        mytype = typecase[match.group(0)]
    return mytype


def get_elements(xmldoc, tag_name, attribute):
    """Returns a list of elements"""
    l = []
    for item in xmldoc.getElementsByTagName(tag_name):
        value = item.getAttribute(attribute)
        l.append(repr(value))
    return l


def get_element(xmldoc, tag_name, attribute):
    for item in xmldoc.getElementsByTagName(tag_name):
        value = item.getAttribute(attribute)
        if len(value) > 0:
            return value
    return None


"""Very simple exception to raise when we found something. For instance to break a loop."""
class Found(Exception):
    pass


class matchresult:
    """Match information"""

    def __init__(self, thefile, theline, thelineno):
        """Represents a match for a keyword.
        Made of a filename and a line"""
        self.file = thefile
        self.line = theline
        self.lineno = thelineno
    
    def __repr__(self):
        return 'file=%s lineno=%d line=%s' % (self.file, self.lineno, self.line)

    def __str__(self):
        return 'file=%s no=%4d line=%30s' % (self.file, self.lineno, self.line)


def recursive_search(search_regexp, directory, exception_list=[], verbose=False):
    """Recursively search in a directory except in some subdirectories
    The exception list actually is a list of regexp for directories.
    
    Returns a dictionary of list of matches:
    match[ keyword ] = [ <'filename', 'matching line content', 'lineno'>,
                         <'filename', 'matching line content', 'lineno'>,
                         <'filename', 'matching line content', 'lineno'>, ]

    We can only have one match per line. Otherwise, this won't work we should be using re.findall
    """
    matches = defaultdict(list)

    if verbose:
        print("Searching in " + directory + " for " + search_regexp.decode('utf-8'))
        print("Exceptions: %s" % (str(exception_list)))

    for entry in os.listdir(directory):
        current_entry = os.path.join(directory, entry)
        try: 
            if os.path.isfile(current_entry):
                for exception in exception_list:
                    # TO DO: not entirely sure we need 'match'? perhaps if it is a regexp?
                    # Remember that "exception" can be part of a path e.g. we want everything that matches blah/bloh
                    # then com/blah/bloh must match
                    match = re.search(exception, current_entry)  # TO DO: not entirely sure that we need the match
                    if match is not None or exception in current_entry:
                        # skip this file
                        raise Found

                # ok, this file must be searched
                lineno = 0
                for line in open(current_entry, 'rb'):
                    lineno += 1
                    match = re.search(search_regexp, line)
                    if match is not None:
                        if verbose:
                            print("Match: File: " + entry + " Keyword: " +
                                  match.group(0).decode('utf-8', errors='replace') +
                                  " Line: " + line.decode('utf-8', errors='replace'))
                        """match.group(0) only provides one match per line if we need more, 
                        re.search is not appropriate
                        and should be replaced by re.findall"""
                        matches[match.group(0).decode('utf-8', errors='replace')].append(matchresult(current_entry, line, lineno))

            if os.path.isdir(current_entry):
                for exception in exception_list:
                    match = re.search(exception, current_entry)
                    if match is not None:
                        # skip this directory
                        raise Found

                # this directory is not in the exception list, we must search it recursively
                try:
                    hismatches = recursive_search(search_regexp, current_entry, exception_list, verbose)
                    # merge in those results
                    for key in hismatches.keys():
                        matches[key].extend(hismatches[key])
                except RuntimeError:
                    # we get this when there are too many recursive dirs
                    pass  # next

        except Found:
            pass  # go to next entry

    return matches



#!/usr/bin/env python3

"""
__author__ = "Axelle Apvrille"
__license__ = "MIT License"
"""
import argparse
import os
import subprocess
import droidutil  # that's my own utilities
import droidsample
import droidreport
import sys
import logging
from droidsql import DroidSql
from droidconfig import generalconfig

property_dump_file = 'details.md'
report_file = 'report.md'
json_file = 'report.json'
__version__ = "3.4.7"

logging.basicConfig(format='%(levelname)s:%(filename)s:%(message)s',
                    level=logging.INFO)


def get_arguments():
    """Read arguments for the program and returns the ArgumentParser"""
    description = 'DroidLysis3 is a Python which processes Android samples. \n'
    'It prepares the sample for analysis, unzipping,'
    'disassembling, decompiling it.'
    'It parses for potentially suspicious features '
    '(e.g "sample roots the phone")'
    parser = argparse.ArgumentParser(description=description,
                                     prog='DroidLysis',
                                     epilog='Version ' + __version__
                                     + ' - Greetz from @cryptax')
    parser.add_argument('-i', '--input',
                        help='input directories or files to process',
                        nargs='+', action='store', default='.')
    parser.add_argument('-o', '--output',
                        help='analysis of input files is written into '
                        'subdirectories of this directory',
                        action='store', default='.')
    parser.add_argument('-c', '--clearoutput',
                        help='erase the output directory at the end. '
                        'Indicates you want something quick.',
                        action='store_true')
    parser.add_argument('-s', '--silent',
                        help='do not display output on console',
                        action='store_true')
    parser.add_argument('-m', '--movein',
                        help='after it has been processed, '
                        'each input file is moved to this directory',
                        action='store')
    parser.add_argument('-v', '--verbose',
                        help='get more detailed messages',
                        action='store_true')
    parser.add_argument('-V', '--version',
                        help='displays version number',
                        action='version',
                        version="%(prog)s "+__version__)
    parser.add_argument('--no-kit-exception',
                        help='by default, ad/dev/stats kits are ruled '
                        'out for searches. '
                        'Use this option to treat them as regular namespaces',
                        action='store_true')
    parser.add_argument('--disable-report',
                        help='do not generate automatic report',
                        action='store_true')
    parser.add_argument('--enable-sql',
                        help='write analysis to SQL database',
                        action='store_true')
    parser.add_argument('--disable-json',
                        help='do not dump analysis to JSON',
                        action='store_true')
    parser.add_argument('--import-exodus',
                        help='import ETIP Exodus Privacy trackers '
                        'and add them to kit config file',
                        action='store_true')
    parser.add_argument('--config',
                        help='general configuration file for DroidLysis. Default: ./conf/general.conf',
                        default='./conf/general.conf',
                        action='store')

    args = parser.parse_args()
    if args.verbose:
        logging.getLogger().setLevel(logging.DEBUG)
    # create output dir if necessary
    droidutil.mkdir_if_necessary(args.output)
    # create movein dir if necessary
    droidutil.mkdir_if_necessary(args.movein)
    return args


def process_input(args):
    """
    Process input.
    Provide ArgumentParser as argument.
    @args args.input contains a list of files and directories to process.
    each file in an input directory are processed, but not recursively.
    each input file is process.
    """
    config = generalconfig(filename=args.config, verbose=args.verbose)
    sql = None
    if args.enable_sql:
        sql = DroidSql()

    for element in args.input:
        if os.path.isdir(element):
            listing = os.listdir(element)
            for file in listing:
                process_file(config,
                             os.path.join(element, file),
                             outdir=args.output,
                             verbose=args.verbose,
                             clear=args.clearoutput,
                             disable_report=args.disable_report,
                             no_kit_exception=args.no_kit_exception,
                             sql=sql,
                             disable_json=args.disable_json,
                             import_exodus=args.import_exodus)
                if args.movein:
                    logging.debug("Moving %s to %s" %
                                  (os.path.join('.', element),
                                   os.path.join(args.movein, element)))
                    # TODO: issue if inner dirs. Are we handling this?
                    try:
                        os.rename(os.path.join(element, file),
                                  os.path.join(args.movein, file))
                    except OSError as e:
                        logging.debug("%s no longer present?: %s\n" % (file, str(e)))

        if os.path.isfile(element):
            process_file(config,
                         os.path.join('.', element),
                         outdir=args.output,
                         verbose=args.verbose,
                         clear=args.clearoutput,
                         disable_report=args.disable_report,
                         silent=args.silent,
                         no_kit_exception=args.no_kit_exception,
                         sql=sql,
                         disable_json=args.disable_json,
                         import_exodus=args.import_exodus
                         )
            if args.movein:
                logging.debug("Moving %s to %s" %
                              (os.path.join('.', element),
                               os.path.join(args.movein, os.path.basename(element))))
                os.rename(os.path.join('.', element), os.path.join(args.movein, os.path.basename(element)))


def process_file(config,
                 infile,
                 outdir='/tmp/analysis',
                 verbose=False,
                 clear=False,
                 disable_report=False,
                 silent=False,
                 no_kit_exception=False,
                 sql=None,
                 disable_json=False,
                 import_exodus=False):
    """Static analysis of a given file"""

    if os.access(infile, os.R_OK):
        if not silent:
            print("Processing file: " + infile + " ...")
        sample = droidsample.droidsample(config=config,
                                         filename=infile,
                                         output=outdir,
                                         verbose=verbose,
                                         clear=clear,
                                         disable_description=disable_report,
                                         silent=silent,
                                         no_kit_exception=no_kit_exception,
                                         import_exodus=import_exodus)
        sample.unzip()
        sample.disassemble()
        sample.extract_file_properties()
        sample.extract_meta_properties()
        sample.extract_manifest_properties()
        sample.extract_dex_properties()
        listofkits = sample.extract_kit_properties()
        if no_kit_exception:
            listofkits = []
        sample.extract_smali_properties(listofkits)
        sample.extract_wide_properties(listofkits)

        if sql is not None:
            sample.properties.write(sql)

        if not disable_json:
            sample.properties.dump_json(os.path.join(sample.outdir, json_file))

        if not silent and not clear:
            report = droidreport.droidreport(sample, console=True, report_to_file=disable_report)
            report.write(os.path.join(sample.outdir, report_file), verbose)

        if not clear:
            analysis_file = open(os.path.join(sample.outdir, property_dump_file), 'a')
            analysis_file.write(str(sample.properties))
            analysis_file.close()
        else:
            if not silent:
                print("Removing directory %s ..." % (sample.outdir))
            proc = subprocess.Popen(['rm', '-rf', sample.outdir])
            proc.communicate()

        sample.close()


def check_python_version():
    if sys.version_info.major < 3:
        print("ERROR: Please run DroidLysis with Python 3")
        quit()


if __name__ == "__main__":
    check_python_version()
    args = get_arguments()
    process_input(args)
    print("END")



import argparse
import configparser

DEFAULTSECT = 'default'

def get_arguments():
    parser =  argparse.ArgumentParser(description="Sort conf files by alphabetic order of sections", prog='sortconf')
    parser.add_argument('-i', '--input', help='Input conf file', action='store', default='./kit.conf')
    parser.add_argument('-o', '--output', help='Output file', action='store', default='./sorted.conf')
    parser.add_argument('-v', '--verbose', help='get more detailed messages', action='store_true')
    args = parser.parse_args()
    return args
    

class OrderedRawConfigParser( configparser.RawConfigParser ):
    """
    Overload standart Class ConfigParser.RawConfigParser
    """
    def __init__( self, defaults = None, dict_type = dict ):
        configparser.RawConfigParser.__init__( self, defaults = None, dict_type = dict )

        def write(self, fp):
            """Write an .ini-format representation of the configuration state."""
            if self._defaults:
                fp.write("[%s]\n" % DEFAULTSECT)
                for key in sorted( self._defaults ):                
                    fp.write( "%s = %s\n" % (key, str( self._defaults[ key ] ).replace('\n', '\n\t')) )
                    fp.write("\n")
            for section in self._sections:
                fp.write("[%s]\n" % section)
                for key in sorted( self._sections[section] ): 
                    if key != "__name__":
                        fp.write("%s = %s\n" %
                                 (key, str( self._sections[section][ key ] ).replace('\n', '\n\t')))    
                fp.write("\n")

def main():
    args = get_arguments()
    parser = OrderedRawConfigParser()
    parser.read(args.input)
    output = open(args.output,'w')
    parser.write(output)
    output.close()
    
    
if __name__ == "__main__":
    main()



"""
__author__ = "Axelle Apvrille"
__status__ = "Alpha"
__license__ = "MIT License"
"""

import sqlalchemy
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, Boolean
import droidconfig


Base = declarative_base()


class Sample(Base):
    __tablename__ = 'samples'

    sha256 = Column(String(64), primary_key=True)
    sanitized_basename = Column(String())
    file_nb_classes = Column(Integer())
    file_nb_dir = Column(Integer())
    file_size = Column(Integer())
    file_small = Column(Boolean())
    filetype = Column(Integer())
    file_innerzips = Column(Boolean())
    manifest_properties = Column(String())
    smali_properties = Column(String())
    wide_properties = Column(String())
    arm_properties = Column(String())
    dex_properties = Column(String())
    kits = Column(String())

    def __repr__(self):
        return "<Sample(sha256=%s)>" % self.sha256


class DroidSql:
    def __init__(self, verbose=False):
        self.verbose = verbose  # to get SQL ALCHEMY requests
        self.engine = sqlalchemy.create_engine(droidconfig.SQLALCHEMY, echo=self.verbose)
        Base.metadata.create_all(self.engine)

    



#!/usr/bin/env python3
# $Source$
"""
__author__ = "Axelle Apvrille"
__license__ = "MIT License"
"""


class droidversion:
    """A small class to hold different versions of Android"""
    def __init__(self, versionstring, apilevel, codename=''):
        self.codename = codename
        self.version = versionstring
        self.apilevel = apilevel

    def __str__(self):
        return "Android %s - %s (API level %d)" % (self.versionstring, self.codename, self.apilevel)


versions = [droidversion('1.0', 1),
            droidversion('1.1', 2),
            droidversion('1.5', 3, 'Cupcake'),
            droidversion('1.6', 4, 'Donut'),
            droidversion('2.0', 5, 'Eclair'),
            droidversion('2.0.1', 6, 'Eclair'),
            droidversion('2.1', 7, 'Eclair'),
            droidversion('2.2', 8, 'Froyo'),
            droidversion('2.3', 9, 'Gingerbread'),
            droidversion('2.3.3', 10, 'Gingerbread'),
            droidversion('3.0', 11, 'Honeycomb'),
            droidversion('3.1', 12, 'Honeycomb'),
            droidversion('3.2', 13, 'Honeycomb'),
            droidversion('4.0', 14, 'Ice Cream Sandwich'),
            droidversion('4.0.3', 15, 'Ice Cream Sandwich'),
            droidversion('4.1', 16, 'Jelly Bean'),
            droidversion('4.2', 17, 'Jelly Bean'),
            droidversion('4.3', 18, 'Jelly Bean'),
            droidversion('4.4', 19, 'KitKat'),
            droidversion('5.0', 21, 'Lollipop'),
            droidversion('5.1', 22, 'Lollipop'),
            droidversion('6.0', 23, 'Marshmallow'),
            droidversion('7.0', 24, 'Nougat'),
            droidversion('7.1', 25, 'Nougat'),
            droidversion('8.0', 26, 'Oreo'),
            droidversion('8.1', 27, 'Oreo'),
            droidversion('9.0', 28, 'Pie'),
            droidversion('10', 29, 'Q'),
            droidversion('11', 30, 'R'),
            droidversion('12.0', 31, 'Snow Cone'),
            droidversion('12.1', 32, 'Snow Cone v2'),
            droidversion('13', 33, 'Tiramisu'),
            droidversion('14', 34, 'Upside Down Cake')
            ]

# ------------------------------------------------------

class droidreport:
    def __init__(self, sample, console=True, report_to_file=True):
        # console means we want to display the result on the console
        # report file means we want to dump in the report file
        self.sample = sample
        self.console = console
        self.report_to_file = report_to_file

    def write(self, report_file, verbose=True):
        if verbose:
            print( "Writing report to " + report_file)
        if self.report_to_file:
            self.reportfile = open(report_file, 'w')
            self.reportfile.write("# %s\n\n" % (self.sample.properties.sha256))

        self.write_properties()

        if self.report_to_file:
            self.reportfile.close()

    def write_file(self, message):
        if self.report_to_file:
            self.reportfile.write(message)

    def write_console(self, message):
        if self.console:
            print(message)

    def write_properties(self):
        # Header / File info
        print("\033[1;36;1m============= Report ============\033[0m")
        self.write_file("{0:20.20}: {1}\n".format('Sanitized basename', self.sample.properties.sanitized_basename))
        self.write_console("{0:20.20}: \033[1;37;1m{1}\033[0m".format('Sanitized basename',
                                                                      self.sample.properties.sanitized_basename))
        
        self.write_file("{0:20.20}: {1}\n".format('SHA256', self.sample.properties.sha256))
        self.write_console("{0:20.20}: \033[1;37;1m{1}\033[0m".format('SHA256', self.sample.properties.sha256))

        self.write_file("{0:20.20}: {1} bytes\n".format('File size', self.sample.properties.file_size))
        self.write_console("{0:20.20}: \033[1;37;1m{1}\033[0m bytes".format('File size',
                                                                            self.sample.properties.file_size))

        self.write_file("{0:20.20}: {1}\n".format('Is small', self.sample.properties.file_small))
        self.write_console("{0:20.20}: \033[1;37;1m{1}\033[0m".format('Is small', self.sample.properties.file_small))

        self.write_file("{0:20.20}: {1}\n".format('Nb of classes', self.sample.properties.file_nb_classes))
        self.write_console("{0:20.20}: \033[1;37;1m{1}\033[0m".format('Nb of classes',
                                                                      self.sample.properties.file_nb_classes))

        self.write_file("{0:20.20}: {1}\n".format('Nb of directories', self.sample.properties.file_nb_dir))
        self.write_console("{0:20.20}: \033[1;37;1m{1}\033[0m".format('Nb of dirs', self.sample.properties.file_nb_dir))


        # Certificate properties
        self.write_file("\nCertificate properties:\n")
        self.write_console("\n\033[0;30;47mCertificate properties\033[0m")
        
        for key in self.sample.properties.certificate.keys():
            if self.sample.properties.certificate[key] is not False:
                self.write_console("{0:20.20}: \033[1;36;1m{1}\033[0m".format(key,
                                                                              self.sample.properties.certificate[key]))
            self.write_file("{0:20.20}: {1}\n".format(key, self.sample.properties.certificate[key]))

        # Manifest properties
        self.write_file("\nManifest properties:\n")
        self.write_console("\n\033[0;30;47mManifest properties\033[0m")
        for key in self.sample.properties.manifest.keys():
            if self.sample.properties.manifest[key] is not False \
                    and self.sample.properties.manifest[key] is not None and self.sample.properties.manifest[key]:
                self.write_file("{0:20.20}: {1}\n".format(key, self.sample.properties.manifest[key]))
                self.write_console("{0:20.20}: \033[1;36;1m{1}\033[0m".format(key,
                                                                              self.sample.properties.manifest[key]))
            else:
                self.write_file("{0:20.20}: {1}\n".format(key,     self.sample.properties.manifest[key]))


        # Smali properties
        self.write_file("\nSmali properties\n")
        self.write_console("\n\033[0;30;47mSmali properties /  What the Dalvik code does\033[0m")
        for section in self.sample.properties.smali.keys():
            if self.sample.properties.smali[section] is not False:
                if (type(self.sample.properties.smali[section]) is list and
                    len(self.sample.properties.smali[section]) > 0) or \
                        (type(self.sample.properties.smali[section]) is bool):
                    self.write_console("{0:20.20}: \033[1;31;1m{1} \033[1;33;40m({2})\033[0m".format(section,
                                                                                                     self.sample.properties.smali[section],
                                                                                                     self.sample.properties.smaliconfig.get_description(section)))
                    self.write_file("{0:20.20}: {1} ({2})\n".format(section,
                                                                    self.sample.properties.smali[section],
                                                                    self.sample.properties.smaliconfig.get_description(section)))
            else:
                self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.smali[section]))

        # Wide properties
        self.write_file("\nWide properties\n")
        self.write_console("\n\033[0;30;47mWide properties /  What Resources/Assets do\033[0m")
        for section in self.sample.properties.wide.keys():
            if self.sample.properties.wide[section] is not False and self.sample.properties.wide[section] is not None \
                    and self.sample.properties.wide[section]:
                if self.sample.properties.wideconfig.get_description(section) is not None:
                    self.write_console("{0:20.20}: \033[1;31;1m{1} \033[1;33;40m({2})\033[0m".format(section,
                                                                                                     self.sample.properties.wide[section],
                                                                                                     self.sample.properties.wideconfig.get_description(section)))
                    self.write_file("{0:20.20}: {1} ({2})\n".format(section, self.sample.properties.wide[section],
                                                                    self.sample.properties.wideconfig.get_description(section)))
                else:
                    self.write_console("{0:20.20}: \033[1;31;1m{1}\033[0m".format(section,
                                                                                  self.sample.properties.wide[section]))
                    self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.wide[section]))
            else:
                # case where the property is False, or None.
                self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.wide[section]))

        # ARM properties
        self.write_file("\nARM properties\n")
        self.write_console("\n\033[0;30;47mARM properties /  What native ARM libraries do\033[0m")
        for section in self.sample.properties.arm.keys():
            if self.sample.properties.arm[section] is not False and self.sample.properties.arm[section] is not None:
                if self.sample.properties.armconfig.get_description(section) is not None:
                    self.write_console("{0:20.20}: \033[1;31;1m{1} \033[1;33;40m({2})\033[0m".format(section,
                                                                                                     self.sample.properties.arm[section],
                                                                                                     self.sample.properties.armconfig.get_description(section)))
                    self.write_file("{0:20.20}: {1} ({2})\n".format(section, self.sample.properties.arm[section],
                                                                    self.sample.properties.armconfig.get_description(section)))
                else:
                    self.write_console("{0:20.20}: \033[1;31;1m{1}\033[0m".format(section, self.sample.properties.arm[section]))
                    self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.arm[section]))
            else:
                # case where the property is False, or None.
                self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.arm[section]))

        # DEX properties
        self.write_file("\nDEX properties\n")
        self.write_console("\n\033[0;30;47mDEX properties /  About classes.dex format\033[0m")
        for section in self.sample.properties.dex.keys():
            if self.sample.properties.dex[section] is not False and self.sample.properties.dex[section] is not None:
                self.write_console("{0:20.20}: \033[1;31;1m{1} \033[0m".format(section,
                                                                               self.sample.properties.dex[section]))
                self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.dex[section]))
            else:
                # case where the property is False, or None.
                self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.dex[section]))
                
        # Kits properties
        self.write_file("\nKit properties\n")
        self.write_console("\n\033[0;30;47mKit properties /  Detected 3rd party SDKs\033[0m")
        for section in self.sample.properties.kits.keys():
            if self.sample.properties.kits[section] is not False and self.sample.properties.kits[section] is not None:
                if self.sample.properties.kitsconfig.get_description(section) is not None:
                    self.write_console("{0:20.20}: \033[1;31;1m{1} \033[1;33;40m({2})\033[0m".format(section,
                                                                                                     self.sample.properties.kits[section],
                                                                                                     self.sample.properties.kitsconfig.get_description(section)))
                    self.write_file("{0:20.20}: {1} ({2})\n".format(section, self.sample.properties.kits[section],
                                                                    self.sample.properties.kitsconfig.get_description(section)))
                else:
                    self.write_console("{0:20.20}: \033[1;31;1m{1}\033[0m".format(section,
                                                                                  self.sample.properties.kits[section]))
                    self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.kits[section]))
            else:
                # case where the property is False, or None.
                self.write_file("{0:20.20}: {1}\n".format(section, self.sample.properties.kits[section]))
        



#!/usr/bin/env python

"""
__author__ = "Axelle Apvrille"
__license__ = "MIT License"
"""
import hashlib
import base64
import os
import re
import sys
import string
import struct
import zlib
import shutil
import subprocess
import xml.dom.minidom
import droidutil
import droidlysis3
import droidconfig
import droidcountry
import droidproperties
import droidziprar
import droidurl
import xml.parsers.expat as expat
import logging

logging.basicConfig(format='%(levelname)s:%(filename)s:%(message)s',
                    level=logging.INFO)


class droidsample:
    # Base class for an Android sample to analyze

    def __init__(self,
                 config,
                 filename,
                 output='/tmp/analysis',
                 verbose=False,
                 clear=False,
                 disable_description=False,
                 silent=False,
                 no_kit_exception=False,
                 import_exodus=False):
        # Setup analysis of a given sample.
        # This does not perform the analysis in itself

        assert filename is not None, "Filename is invalid"

        self.config = config
        self.absolute_filename = filename
        self.clear = clear
        # we need those for recursive calls to process_file
        self.disable_description = disable_description
        self.silent = silent
        self.no_kit_exception = no_kit_exception
        self.ziprar = None  # zip file or rar file file handle
        self.verbose = verbose
        if verbose:
            logging.getLogger().setLevel(logging.DEBUG)

        sanitized_basename = droidutil.sanitize_filename(
            os.path.basename(filename))

        self.properties = droidproperties.droidproperties(
            config=self.config,
            samplename=sanitized_basename,
            sha256=droidutil.sha256sum(filename),
            verbose=verbose,
            import_exodus=import_exodus)
        logging.debug("SHA256: %s" % (self.properties.sha256))

        """Computing the SHA1 of a file is only useful to help out the analyst.
        The digest is written to the automatic analysis/description
        of the sample. That description is located in the output analysis
        directory, so obviously, if the end-user specifies he wants
        the output analysis directory erased, he obviously
        does not want to know about the SHA1 digest, and
        thus it's useless to compute it."""
        if not clear:
            sha1 = droidutil.sha1sum(filename)
            logging.debug("SHA1: %s" % (sha1))

        # Creating the output analysis directory
        self.outdir = os.path.join(output, '{filename}-{hash}'.format(
                filename=sanitized_basename,
                hash=self.properties.sha256))

        logging.debug("Output dir: " + self.outdir)

        if os.path.exists(self.outdir):
            try:
                shutil.rmtree(self.outdir,
                              ignore_errors=droidutil.on_rm_tree_error)
                os.makedirs(self.outdir)
            except RuntimeError:
                logging.debug("Failed to remove directory RuntimeError")
        else:
            os.makedirs(self.outdir)

        # standard output for shell commands
        if self.verbose:
            self.process_output = None
        else:
            self.process_output = open("/dev/null", 'w')

    def close(self):
        if self.process_output is not None:
            self.process_output.close()

    def unzip(self):
        """
        This method will unzip/unrar the sample,
        and recursively unzip/unrar inner zips/rars.
        If we are not removing the analysis directory
        (clearoutput option), then we also
        unzip the sample in outdir/unzipped subdirectory.
        If the sample is password protected,
        we try 'infected' as password.
        Returns the file type of the sample: droidutil
        <FILE CONSTANT> (UNKNOWN, APK, DEX, ...)
        """
        logging.debug("------------- Unzipping %s" % (self.absolute_filename))

        self.properties.filetype = droidutil.get_filetype(
            self.absolute_filename)

        if self.properties.filetype == droidutil.ARM or \
           self.properties.filetype == droidutil.UNKNOWN or \
           self.properties.filetype == droidutil.DEX:
            logging.debug("This is a %s. Nothing to unzip for %s"
                          % (droidutil.str_filetype(self.properties.filetype),
                             self.absolute_filename))
            return self.properties.filetype

        if self.properties.filetype == droidutil.ZIP or \
                self.properties.filetype == droidutil.RAR:
            if self.properties.filetype == droidutil.ZIP:
                self.ziprar = droidziprar.droidziprar(
                    self.absolute_filename,
                    zipmode=True, verbose=self.verbose)
            else:
                self.ziprar = droidziprar.droidziprar(
                    self.absolute_filename,
                    zipmode=False, verbose=self.verbose)
            if self.ziprar.handle is None:
                self.properties.filetype = droidutil.UNKNOWN  # damaged zip/rar
                logging.debug("Unable to unzip/unrar %s because of errors"
                              % self.absolute_filename)
                return droidutil.UNKNOWN
            # Now, we know self.ziprar is valid and open.
            self.properties.filetype, innerzips = self.ziprar.get_type()
            if innerzips:
                self.properties.file_innerzips = True
                logging.debug("There are inner zips/rars in "
                              + self.absolute_filename)

                for element in innerzips:
                    # extract the inner zip/rar
                    logging.debug("Extracting " + element
                                  + " inside " + self.absolute_filename)
                    try:
                        self.ziprar.extract_one_file(element, self.outdir)
                        logging.debug("Recursively processing "
                                      + os.path.join(self.outdir, element))
                        # TODO: this is probably buggy: we
                        # should be providing enable_sql too etc.
                        droidlysis3.process_file(
                            os.path.join(self.outdir, element),
                            self.outdir, self.verbose, self.clear,
                            self.disable_description,
                            self.no_kit_exception)
                    except:
                        logging.warning("Cannot extract %s : %s"
                                        % (element, sys.exc_info()[0]))

        if self.properties.filetype == droidutil.APK:
            # our zip actually is an APK
            if not self.clear:
                # let's unzip
                logging.debug("Unzipping " +
                              self.absolute_filename
                              + " to " + os.path.join(self.outdir, 'unzipped'))
                try:
                    self.ziprar.extract_all(
                        outdir=os.path.join(self.outdir, 'unzipped'))
                except:
                    logging.warning("Unzipping failed (catching exception): %s"
                                    % (sys.exc_info()[0]))

        return self.properties.filetype

    def disassemble(self):
        """
        Disassembles the sample (as much as possible),
        and if the end-user is interested in output (clear unset)
        also decompiles it (as much as possible).

        APK: 
                java -jar apktool.jar [-q] d file.apk outdir/apktool
                if apktool failed
                      java -jar baksmali.jar -o outdir/smali classes.dex in file.apk
                if clear unset,
                      unzip file.apk -d outdir/unzipped
                else
                      just unzip META-INF/

        DEX:
                java -jar baksmali.jar -o outdir/smali classes.dex
                if clear unset,
                      make sure classes.dex is readable
                      dex2jar classes.dex -o classes-dex2jar.jar
                      unzip -qq classes-dex2jar.jar -d outdir/unjarred

        What we'll find:
        APK, clear unset: ./smali AndroidManifest.xml ./unzipped classes.dex classes-dex2jar.jar ./unjarred 
        APK, clear set  : ./smali AndroidManifest.xml classes.dex

        DEX, clear unset: classes.dex ./smali
        DEX, clear set  : classes.dex ./smali classes-dex2jar.jar ./unjarred

        """
        logging.debug("------------- Disassembling")

        if self.properties.filetype == droidutil.ARM or \
                self.properties.filetype == droidutil.RAR or \
                self.properties.filetype == droidutil.CLASS or \
                self.properties.filetype == droidutil.UNKNOWN:
            logging.debug("Nothing to disassemble for "
                          + self.absolute_filename)
            return

        if self.properties.filetype == droidutil.APK:
            # APKTOOL won't output to an existing
            # dir unless you use the -f switch. But then, -f erases the contents
            # of the output dir... So we can't do this directly on self.outdir
            apktool_outdir = os.path.join(self.outdir, "apktool")
            logging.debug("Running apktool on inputfile="
                          + self.absolute_filename)

            if self.verbose:
                logging.debug("Apktool command: java -jar %s d -f %s %s"
                              % (self.config.APKTOOL_JAR,
                                 self.absolute_filename, apktool_outdir))
                subprocess.call(["java", "-jar", self.config.APKTOOL_JAR,
                                 "d", "-f", self.absolute_filename,
                                 "-o", apktool_outdir])
            else:
                # with quiet option
                subprocess.call(["java", "-jar", self.config.APKTOOL_JAR,
                                 "-q", "d", "-f", self.absolute_filename,
                                 "-o", apktool_outdir],
                                 stdout=self.process_output,
                                 stderr=self.process_output)

            if os.path.isdir(apktool_outdir):
                droidutil.move_dir(apktool_outdir, self.outdir)

            # we want all smali_classes? dir in smali
            if os.path.exists(os.path.join(self.outdir, "smali_classes2")):
                # we have multidex - we are going to move
                # all smali classes in the same directory
                logging.debug("Moving multidex smali classes to ./smali")

                os.system("cp -R "
                          + os.path.join(self.outdir, "./smali_classes*/*")
                          + " " + os.path.join(self.outdir, "./smali"))
                os.system("rm -r "
                          + os.path.join(self.outdir, "./smali_classes*"))
                
            if self.verbose:
                logging.debug("Apktool finished")

            # extract classes.dex whatever happens, we'll use it
            logging.debug("Extracting classes*.dex")
            try:
                self.ziprar.extract_one_file('classes.dex', self.outdir)
                for i in range(2, 1000):
                    self.ziprar.extract_one_file('classes{}.dex'.format(i), self.outdir)
                    if not os.path.exists(os.path.join(self.outdir, 'classes{}.dex'.format(i))):
                        break
                    self.properties.smali['multidex'].append('classes{}.dex'.format(i))
                    
            except:
                logging.debug("Extracting classes.dex failed: %s" % (sys.exc_info()[0]))
                logging.warning("Extracting classes.dex failed")

        # Disassemble the DEX(es)
        dex_files = []
        if self.properties.filetype == droidutil.DEX:
            dex_files.append(self.absolute_filename)
        else:
            if len(self.properties.smali['multidex']) > 0:
                for d in self.properties.smali['multidex']:
                    dex_files.append(os.path.join(self.outdir, d))
            else:
                dex_files.append(os.path.join(self.outdir, 'classes.dex'))

        smali_dir = os.path.join(self.outdir, 'smali')

        if (not os.access(smali_dir, os.R_OK) or
                (os.access(smali_dir, os.R_OK) and not os.listdir(smali_dir))):
            # we only do this if apktool didn't baksmali correctly
            for d in dex_files:
                if os.access(d, os.R_OK):
                    logging.debug("Baksmali on {} -> {}".format(d, smali_dir))
                    try:
                        subprocess.call(["java", "-jar", self.config.BAKSMALI_JAR,
                                         "d", "-o", smali_dir, d],
                                        stdout=self.process_output, stderr=self.process_output)
                    except:
                        logging.warning("[-] Baksmali failed for {}".format(d))

        # Decompile the DEX(es)
        logging.debug("------------- Decompiling")

        if not self.clear:
            for d in dex_files:
                if os.access(d, os.R_OK):
                    jar_file = os.path.join(self.outdir, '{}-dex2jar.jar'.format(
                        os.path.splitext(os.path.basename(d))[0]))
                    if os.access(self.config.DEX2JAR_CMD, os.X_OK):
                        if self.verbose:
                            logging.debug("Dex2jar on " + d)
                            try:
                                subprocess.call([self.config.DEX2JAR_CMD, "--force", d, "-o", jar_file],
                                                stdout=self.process_output, stderr=self.process_output)
                            except:
                                logging.warning("[-] Dex2jar failed on {}".format(d))
                    else:
                        logging.warning("Dex2jar software is not executable, skipping (file: {0})".format(self.config.DEX2JAR_CMD))
                    
                    if os.access(jar_file, os.R_OK):
                        logging.debug("Unjarring " + jar_file)
                        jarziprar = droidziprar.droidziprar(jar_file, zipmode=True, verbose=self.verbose)
                        if jarziprar.handle is None:
                            logging.debug("Bad Jar / Failed to unjar " + jar_file)
                        else:
                            try:
                                jarziprar.extract_all(os.path.join(self.outdir, 'unjarred'))
                            except:
                                logging.warning("Failed to unjar: %s" % (sys.exc_info()[0]))
                        jarziprar.close()

        # Convert binary Manifest
        if self.properties.filetype == droidutil.APK:
            manifest = os.path.join(self.outdir, 'AndroidManifest.xml')
            if not os.access(manifest, os.R_OK) or os.path.getsize(manifest) == 0:
                try:
                    logging.debug( "Extracting binary AndroidManifest.xml")
                    self.ziprar.extract_one_file('AndroidManifest.xml', self.outdir)
                    if os.access( manifest, os.R_OK) and os.path.getsize(manifest) > 0:
                        textmanifest = os.path.join( self.outdir, 'AndroidManifest.text.xml')
                        subprocess.call( [ "androaxml.py", "--input", manifest, \
                                           "--output", textmanifest ], \
                                         stdout=self.process_output, stderr=self.process_output)
                        if os.access( textmanifest, os.R_OK ):
                            # overwrite the binary manifest with the converted text one
                            os.rename(textmanifest, manifest)
                except AssertionError:
                    logging.warning("Failed to extract binary manifest: %s" % (sys.exc_info()[0]))
                
                except FileNotFoundError:
                    logging.warning("Error while unzipping or Androguard not installed (no androaxml.py)")
                     
    def extract_file_properties(self):
        """Extracts file size, 
        nb of dirs and classes in smali dir"""
        logging.debug("------------- Extracting file properties")
            
        self.properties.file_size = os.stat(self.absolute_filename).st_size

        if self.properties.file_size < 70000:
            self.properties.file_small = True
        
        smali_dir = os.path.join(self.outdir, "smali")

        if os.access(smali_dir, os.R_OK):
            self.properties.file_nb_dir, self.properties.file_nb_classes = droidutil.count_filedirs(smali_dir)

        logging.debug("Filesize: %d" % (self.properties.file_size))
        logging.debug("Is Small: %d" % (self.properties.file_small))
        logging.debug("Nb Class: %d" % (self.properties.file_nb_classes))
        logging.debug("Nb Dir  : %d" % (self.properties.file_nb_dir))

    def extract_meta_properties(self):
        """Extracting meta-data related to APK's signature timestamp and signing certificate"""
        if self.properties.filetype == droidutil.APK:
            self.properties.certificate['timestamp'] = self.ziprar.get_date('META-INF/MANIFEST.MF')
            if self.properties.certificate['timestamp'] != None:
                self.properties.certificate['year'] = self.properties.certificate['timestamp'][0]
                logging.debug("APK timestamp: %d" % (self.properties.certificate['timestamp'][0]))

            # extract properties from the certificate
            logging.debug("------------- Extracting properties from certificate")

            list = []
            try:
                list = self.ziprar.extract_pattern(self.outdir, 'META-INF/.*\.RSA')
                if not list:
                    list = self.ziprar.extract_pattern(self.outdir, 'META-INF/.*\.DSA')
            except:
                logging.debug("Error at extraction: %s" % (sys.exc_info()[0]))

            if list:
                try: 
                    keyout = subprocess.check_output([self.config.KEYTOOL, "-printcert",  "-file",
                                                      os.path.join(self.outdir, list[0])],
                                                      stderr=self.process_output).decode('utf-8')
                    keyout = re.sub(': ', '#', keyout)
                    keyout = re.sub('\t| ', '', keyout)
                    keyout = re.sub('until#', '\nUntil#', keyout)
                    keysplit = re.split('#|\n', keyout)
                    try:
                        index = keysplit.index("Owner")
                        self.properties.certificate['owner'] = keysplit[index+1]
                        logging.debug("Certificate Owner: "+self.properties.certificate['owner'])
                        self.extract_certificate_owner_properties(self.properties.certificate['owner'])
                            
                    except ValueError:
                        logging.debug("Certificate Owner not present")
                    try:
                        index = keysplit.index("Serialnumber")
                        self.properties.certificate['serialno'] = keysplit[index+1]
                        logging.debug("Certificate Serial no: "+self.properties.certificate['serialno'])

                        # detect typical dev certificate
                        if re.search('936eacbe07f201df', self.properties.certificate['serialno'], re.IGNORECASE):
                            self.properties.certificate['dev'] = True
                            logging.debug("Dev certificate detected")
                    except ValueError:
                        logging.debug("Serial number not present")
                    try:
                        index = keysplit.index("Signaturealgorithmname")
                        self.properties.certificate['algo'] = keysplit[index+1]
                        logging.debug("Algo: "+self.properties.certificate['algo'])
                    except ValueError:
                        logging.debug("Signature algorithm name not present")
                except subprocess.CalledProcessError as e:
                    logging.debug("Caught CalledProcessError: ", e.output)
                    logging.debug("Probably an invalid certificate")
            else:
                logging.debug("No certificate found")

    def extract_certificate_owner_properties(self, owner):
        # owner should not be null
        m = re.search('C=(\w*)', owner, re.IGNORECASE)
        if m is not None:
            cert_country = m.group(0)[2:]
            if re.search('Unknown', cert_country, re.IGNORECASE):
                self.properties.certificate['unknown_country'] = True
            else:
                self.properties.certificate['country'] = droidcountry.to_int(cert_country)
                logging.debug("Certificate Country = %s (%d)" % (cert_country, self.properties.certificate['country']))

        # Debug certificate
        m = re.search('OU=Android  O=Android L=Mountain View', owner, re.IGNORECASE)
        if m is not None:
            self.properties.certificate['debug'] = True
            
        # AV owner
        av_list = ('O=www.netqin.com',
                   'OU=Symantec',
                   'CN=Fortinet Android  OU=Android  O=Fortinet  L=Burnaby  ST=British Columbia  C=BC',
                   'OU=Engineering  O=Webroot Software Inc  L=Boulder  ST=Colorado  C=US|CN=James Burgess',
                   'OU=Mobile Security, O=Flexilis, L=Los Angeles, ST=CA, C=US|O=Sophos Ltd.  L=Abingdon  C=UK',
                   'OU=Application Development  O=G Data Software AG  L=Bochum  ST=NRW  C=DE',
                   'CN=VIRUSTOTAL  OU=VIRUSTOTAL  O=VIRUSTOTAL  L=Malaga  ST=Malaga  C=ES',
                   'CN=Qihoo  OU=Qihoo 360 Technology Co Ltd  O=Qihoo 360 Technology Co Ltd  L=Chaoyang  ST=Beijing  C=CN',
                   'CN=KAIDI,OU=KAIDI,O=KAIDI,L=Beijing,ST=Beijing,C=86',
                   'EMAILADDRESS=android\@avast.com  CN=avast! Android  O=AVAST Software a.s.  L=Prague  ST=Prague  C=CZ')

        for av in av_list:
            m = re.search(av, owner, re.IGNORECASE)
            if m is not None:
                if self.verbose:
                    print("Certificate owner matches AV : "+m.group(0))
                self.properties.certificate['av'] = True

        # Companies
        famous_list = ('CN=Android  OU=Android  O=Google Inc\.  L=Moutain View  ST=California  C=US',
                       'O=Rovio Mobile Ltd  L=Helsinki  C=FI',
                       'CN=Facebook Corporation  OU=Facebook  O=Facebook Mobile  L=Palo Alto  ST=CA  C=US')
        for f in famous_list:
            m = re.search(f, owner, re.IGNORECASE)
            if m is not None:
                logging.debug("Certificate owner matches Famous : "+m.group(0))
                self.properties.certificate['famous'] = True

    def is_packed(self):
        """
        We assume a sample is packed if the main activity its manifest references cannot be found in the DEX
        + DexClassLoader or Dex File
        This test is far from perfect
        """
        smali_dir = os.path.join(self.outdir, 'smali')
        missing = False
        if self.properties.manifest['main_activity'] is not None:
            filename = os.path.join(smali_dir, self.properties.manifest['main_activity'].replace('.', os.path.sep)
                                    .replace('\'', '') + '.smali')
            if not os.access(filename, os.R_OK):
                logging.debug("Unable to find Main Activitity: {} filename={}".format(self.properties.manifest['main_activity'],
                                                                                      filename))
                missing = True

        if missing and (self.properties.smali['dex_class_loader'] or self.properties.smali['dex_file'] or self.properties.smali['load_library'] or self.properties.smali['class_loader']):
            self.properties.smali['packed'] = True

    def extract_manifest_properties(self):
        """Extracting services, receivers, activities etc from manifest"""
        manifest = os.path.join(self.outdir, 'AndroidManifest.xml')
        if self.properties.filetype == droidutil.APK and os.access(manifest, os.R_OK) and os.path.getsize(manifest)>0:
            try:
                xmldoc = xml.dom.minidom.parse(manifest)
            except expat.ExpatError:
                logging.debug("XML parsing error")
                return

            # getting package's name
            self.properties.manifest['package_name'] = xmldoc.documentElement.getAttribute('package')

            tab = droidutil.get_elements(xmldoc, 'service', 'android:name')
            tab.extend(droidutil.get_elements(xmldoc, 'service', 'obfuscation:name'))
            for t in tab:
                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
                if name != "''":
                    self.properties.manifest['services'].append(name)
                                                            
            tab = droidutil.get_elements(xmldoc, 'receiver', 'android:name')
            tab.extend(droidutil.get_elements(xmldoc, 'receiver', 'obfuscation:name'))
            for t in tab:
                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
                if name != "''":
                    self.properties.manifest['receivers'].append(name)

            tab = droidutil.get_elements(xmldoc, 'activity', 'android:name')
            tab.extend(droidutil.get_elements(xmldoc, 'activity', 'obfuscation:name'))
            for t in tab:
                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
                if name != "''":
                    self.properties.manifest['activities'].append(name)

            tab = droidutil.get_elements(xmldoc, 'provider', 'android:name')
            tab.extend(droidutil.get_elements(xmldoc, 'provider', 'obfuscation:name'))
            for t in tab:
                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
                if name != "''":
                    self.properties.manifest['providers'].append(name)

            tab = droidutil.get_elements(xmldoc, 'uses-library', 'android:name')
            for t in tab:
                self.properties.manifest['libraries'].append(re.sub(r"u'(?P<nom>.*)'", r"\g<nom>", t))

            tab = droidutil.get_elements(xmldoc, 'uses-permission', 'android:name')
            tab.extend(droidutil.get_elements(xmldoc, 'uses-permission', 'obfuscation:name'))
            for t in tab:
                name = t.replace('android.permission.','').replace("u'", '').replace("'", '')
                if name != '':
                    self.properties.manifest['permissions'].append(name)

            tab = droidutil.get_elements(xmldoc, 'application', 'obfuscation:name')
            tab.extend(droidutil.get_elements(xmldoc, 'application', 'android:name'))
            for t in tab:
                if 'multidex' in t:
                    self.properties.manifest['multidex'] = True
                    break

            self.properties.manifest['maxSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:maxSdkVersion')
            self.properties.manifest['minSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:minSdkVersion')
            self.properties.manifest['targetSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:targetSdkVersion')
            tab = droidutil.get_elements(xmldoc, 'meta-data', 'android:name')
            if self.verbose:
                logging.debug('meta-data: ' + ''.join(tab))
            for t in tab:
                if 'android.content.ContactDirectory' in t:
                    self.properties.manifest['auto_load'] = True
                    if self.verbose:
                        logging.debug('Suspicious ContactDirectory meta-data trick')
                    break

            if self.verbose:
                logging.debug("MinSDK=%s MaxSDK=%s TargetSDK=%s" % (self.properties.manifest['minSDK'],
                                                                    self.properties.manifest['maxSDK'],
                                                                    self.properties.manifest['targetSDK']))
                for perm in self.properties.manifest['permissions']:
                    logging.debug("Requires permission " + perm)

            # get main activity
            for actitem in xmldoc.getElementsByTagName('activity'):
                for a in actitem.getElementsByTagName('action'):
                    if a.getAttribute( 'android:name') == 'android.intent.action.MAIN' \
                            or a.getAttribute('obfuscation:name') ==  'android.intent.action.MAIN':
                        for b in actitem.getElementsByTagName('category'):
                            if b.getAttribute('android:name') == 'android.intent.category.LAUNCHER'\
                                    or b.getAttribute('obfuscation:name') ==  'android.intent.category.LAUNCHER':
                                if actitem.getAttribute('android:name') != '':
                                    self.properties.manifest['main_activity'] = actitem.getAttribute('android:name')
                                else:
                                    self.properties.manifest['main_activity'] = actitem.getAttribute('obfuscation:name')
            if self.verbose and self.properties.manifest['main_activity'] is not None:
                logging.debug("Main activity: " + self.properties.manifest['main_activity'])

            # search for swf
            metalist = droidutil.get_elements(xmldoc, 'meta-data', 'android:value')
            for item in metalist:
                if re.search('\.swf', item):
                    self.properties.manifest['swf'] = True

            # get sms listener
            for r in xmldoc.getElementsByTagName('receiver'):
                for i in r.getElementsByTagName('intent-filter'):
                    for a in i.getElementsByTagName('action'):
                        if a.getAttribute('android:name') == 'android.provider.Telephony.SMS_RECEIVED':
                            self.properties.manifest['listens_incoming_sms'] = True
                        if a.getAttribute('android:name') == 'android.intent.action.NEW_OUTGOING_CALL':
                            self.properties.manifest['listens_outgoing_call'] = True

            logging.debug("Listens to incoming SMS  : %d" % (self.properties.manifest['listens_incoming_sms']))
            logging.debug("Listens to outgoing calls: %d" % (self.properties.manifest['listens_outgoing_call']))

    def extract_kit_properties(self):
        """
        Detects which kits are present in the sample currently analyzed
        Returns something like: ['apperhand', 'jackson', 'applovin', 'leadbolt', 'airpush']
        """
        list = []
        if self.properties.filetype == droidutil.APK or self.properties.filetype == droidutil.DEX:
            smali_dir = os.path.join(self.outdir, "smali")
            if os.access(smali_dir, os.R_OK):
                for section in self.properties.kitsconfig.get_sections():
                    pattern_list = self.properties.kitsconfig.get_pattern(section).split('|')
                    for pattern in pattern_list:
                        for root, dirs, fname in os.walk(smali_dir):
                            if pattern in root:
                                logging.debug("kits[%s] = True (detected pattern: %s)" % (section, pattern))
                                list.append(section)
                                self.properties.kits[section] = True
                                break # break one level
                        if self.properties.kits[section] is True:
                            break  # break another level

        return list

    def extract_dex_properties(self):
        """Extracts information at DEX level. Requires read access to the DEX file. """
        if self.properties.filetype == droidutil.APK or self.properties.filetype == droidutil.DEX:
            if self.properties.filetype == droidutil.DEX:
                dex_file = self.absolute_filename
            else:
                dex_file = os.path.join(self.outdir, 'classes.dex')

            if os.access(dex_file, os.R_OK) and os.stat(dex_file).st_size > 0:
                file = open(dex_file, 'rb')
                magic = file.read(8)
                self.properties.dex['magic_unknown'] = True
                if magic[0:3] == 'dey':
                    self.properties.dex['odex'] = True
                for i in range(35, 39):
                    if magic[4:7] == (b'0%d' % i):
                        self.properties.dex['magic'] = i
                        self.properties.dex['magic_unknown'] = False

                logging.debug("DEX Magic: %s" % (repr(magic)))

                checksum = file.read(4)
                sha1 = file.read(20)

                # check sha1 of file
                if sha1 != '':
                    computed_sha1 = hashlib.sha1(file.read()).hexdigest()
                    if sha1.hex() != computed_sha1:
                        self.properties.dex['bad_sha1'] = True

                        logging.debug("DEX SHA1 read    : %s" % sha1.hex())
                        logging.debug("DEX SHA1 computed: %s" % computed_sha1)
                else:
                    logging.debug("Impossible to read file's SHA1 => impossible to check")

                # check checksum
                if checksum != '':
                    file.seek(8+4, 0)  # 0 = from beginning, skip magic and checksum
                    computed_adler32 = zlib.adler32(file.read()) & 0xffffffff  # beware, adler32 returns an INTEGER
                    if computed_adler32 != struct.unpack("<I", checksum)[0]:  # converting both numbers to integers
                        self.properties.dex['bad_adler32'] = True

                    logging.debug("DEX checksum read    : %s (reverse order)" % (checksum.hex()))
                    logging.debug("DEX checksum computed: %s" % (hex(computed_adler32)))
                else:
                    logging.debug("Impossible to read file's checksum => impossible to check")

                # check header size
                file.seek(8+4+20+4, 0)
                header_size = struct.unpack("<I", file.read(4))[0]
                if header_size > 0x70:
                    logging.debug("DEX header is bigger than expected: %d (HoseDex2Jar?)" % header_size)
                    self.properties.dex['big_header'] = True

                # look for 0 0x0 if-eq v0, v0, +9
                # 1 0x4 fill-array-data v0, +3 (0x7)
                # 2 0xa fill-array-data-payload
                # See http://www.dexlabs.org/blog/bytecode-obfuscation
                file.seek(0x68, 0)
                data_size = struct.unpack("<I", file.read(4))[0]
                data_offset = struct.unpack("<I", file.read(4))[0]
                file.seek(data_offset, 0)
                buffer = file.read(data_size)
                fill_array_pattern = re.compile(b"\x32\x00\x09\x00\x26\x00\x03\x00\x00\x00")
                match = fill_array_pattern.search(buffer)
                if match is not None:
                    logging.debug("fill-array-data trick located at offset=%d" % (data_offset + match.start()))
                    # to print the matching string: ''.join( [ "%02X " % ord( x ) for x in buffer[match.start():match.end() ] ).strip()
                    self.properties.dex['thuxnder'] = True
                
                file.close()            
            else:
                logging.debug("Dex file %s is missing or empty" % dex_file)

    def extract_smali_properties(self, list_of_kits):
        if self.properties.filetype != droidutil.APK and\
                self.properties.filetype != droidutil.DEX:
            # no smali properties to extract in that type of file
            return

        logging.debug("------------- Extracting Smali properties")

        smali_dir = os.path.join(self.outdir, 'smali')
        if os.access(smali_dir, os.R_OK) and os.listdir(smali_dir) != []: 
            exceptions = []
            for kit in list_of_kits:
                pattern_list = self.properties.kitsconfig.get_pattern(kit).split('|')
                for pattern in pattern_list:
                    if pattern is not None and pattern != '':
                        exceptions.append(pattern)  # pattern may be part of a path so do not prefix with smali_dir
                    else:
                        logging.debug("WARNING: configuration file error: empty pattern for %s" % kit)
                              
            smali_regexp = self.properties.smaliconfig.get_all_regexp()
            match = droidutil.recursive_search(smali_regexp, smali_dir, exceptions, False)
            
            analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
            analysis_file.write('# Smali keywords\n')
            analysis_file.close()

            self.properties.smaliconfig.match_properties(match, self.properties.smali)
            for mykey in match.keys():
                '''if self.verbose:
                    for element in match[mykey]:
                        print("- keyword=%s detected: %s" % (mykey, str(element)))'''
                if mykey.find('android_id') >= 0 and match[mykey]:
                    # this matches const-string v[0-9]*, "android_id"/'
                    self.properties.smali['android_id'] = True

                if mykey.find('scp') >= 0 and mykey.find('const-string') >= 0 and match[mykey]:
                    # const-string v[0-9]*, ".*scp.*"
                    self.properties.smali['scp'] = True

                if mykey.find('ssh') >= 0 and mykey.find('const-string') >= 0 and match[mykey]:
                    # const-string v[0-9]*, ".*ssh.*'
                    self.properties.smali['ssh'] = True

                analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
                # let's not dump for nops
                if not (mykey == ' nop'):
                    if match[mykey]:
                        analysis_file.write("## %s\n" % mykey)
                    for element in match[mykey]:
                        append = self.extract_method_name(str(element))
                        analysis_file.write("- "+append+str(element)+"\n")
                    analysis_file.write('\n')
                    analysis_file.close()

            # test if sample is likely to be packed
            self.is_packed()
        else:
            logging.debug("Cannot extract smali properties, because directory %s not found" % smali_dir)
            # all smali properties should then be set to unknown
            for key in sorted(self.properties.smali.keys()):
                self.properties.smali[key] = 'unknown'

    def extract_method_name(self, element):
        m = re.findall(r'file=(.*?)\s+no=\s*(\d+)', element)
        if m:
            file_path, line_no = m[0]
            with open(file_path, 'r') as f:
                lines = f.readlines()
                for i in range(int(line_no), 0, -1):
                    line = lines[i]
                    if line.startswith('.method'):
                        clazz = (lines[0].split(' ')[-1]).strip()
                        method = (line.split(' ')[-1]).strip()
                        return f'path={clazz}->{method} '
        return ''

    def extract_wide_properties(self, list_of_kits):
        """Will look for given properties (e.g GPS usage, presence of executables
        in all subdirectories (smali, assets, resources, library...)"""

        logging.debug("------------- Extracting Wide properties")

        # detecting presence of ijiami packer
        ijiami = os.path.join(self.outdir, 'unzipped/assets/ijiami.dat')
        if os.access(ijiami, os.R_OK):
            self.properties.wide['ijiami'] = True

        # detecting Flutter debug mode
        flutter_debug = os.path.join(self.outdir, 'unzipped/assets/flutter_assets/kernel_blob.bin')
        if os.access(flutter_debug, os.R_OK):
            self.properties.kits['flutter'] = True
        
        # detecting react native framework using index.android.bundle
        react_assets = os.path.join(self.outdir, 'unzipped/assets/index.android.bundle')
        if os.access(react_assets, os.R_OK):
            self.properties.wide['react_asset'] = True
        
        # Run "file" to detect hermes bytecode version
        proc = subprocess.Popen(['file', react_assets], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output = proc.communicate()[0]

        analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
        analysis_file.write('# Hermes Bytecode version\n')
        analysis_file.write(output.decode() + '\n')
        analysis_file.close()

        # detect executables in resources
        self.find_exec_in_resources()

        # other properties
        if self.properties.filetype == droidutil.APK or self.properties.filetype == droidutil.DEX:
            exceptions = []
            for kit in list_of_kits:
                pattern_list = self.properties.kitsconfig.get_pattern(kit).split('|')
                for pattern in pattern_list:
                    exceptions.append(pattern)
            exceptions.append("/unjarred")
            exceptions.append("/unzipped")
            exceptions.append("/unknown")
            exceptions.append("classes.dex")
            exceptions.append('details.md')

            wide_regexp = self.properties.wideconfig.get_all_regexp()
            match = droidutil.recursive_search(wide_regexp, self.outdir, exceptions, False)
            analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
            analysis_file.write('# Keywords in resources, assets, lib\n\n')
            analysis_file.close()

            self.properties.wideconfig.match_properties(match, self.properties.wide)

            for mykey in match.keys():  # remember mykey is the matched value not the pattern
                if match[mykey]:
                    analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
                    analysis_file.write("- "+str(mykey)+"\n")
                    analysis_file.close()

                # put here keywords for which you need to do some processing on each match

                # keep a list of phonenumbers we detect
                # mykey will be the phonenumber
                # International phone number detector
                # actually, it should be \+[0-9]{1,3}[0-9]{1,14} but that
                # generates too many false positives (numbers for other reasons)
                # so I'm being conservative
                if mykey.startswith('+') and self.properties.wide['has_phonenumbers']:  # mykey.startswith('+') and
                    if re.search(b"\+[0-9]{1,3}[0-9]{10,14}", mykey):
                        self.properties.wide['phonenumbers'].append(mykey)
                        logging.debug("Phone number spotted: " + mykey)
                        analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
                        analysis_file.write("- "+str(mykey)+"\n")
                        analysis_file.close()

                # we want to process each potential URL and IP address
                ip_pattern = re.compile("\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}")
                if ip_pattern.match(mykey):
                    logging.debug("IP address to check: {}".format(mykey))
                    self.interesting_url(mykey)
                    
                if mykey.find('http') >= 0 and match[mykey]:
                    self.interesting_url(mykey)

            analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
            analysis_file.write('\n\n')
            analysis_file.close()

            # detect base64 encoded string in resources
            self.find_base64_strings(self.outdir, exceptions, self.properties.wide['base64_strings'])

            # trying to grab the application's name
            if not self.clear:
                strings_file = os.path.join(self.outdir, "res/values/strings.xml")
                if os.access(strings_file, os.R_OK):
                    xmldoc = xml.dom.minidom.parse(strings_file)
                    sitems = xmldoc.getElementsByTagName('string')
                    if sitems is None:
                        for item in sitems:
                            if item.hasAttributes():
                                if item.getAttribute('name') == 'app_name':
                                    if item.hasChildNodes():
                                        self.properties.app_name = item.childNodes[0].data
                                        # rule out chinese characters: TODO: convert chinese characters to printable
                                        self.properties.app_name = re.sub('[^:print:]', '', self.properties.app_name)
                                        break

    def extract_arm_properties(self, arm_filename=''):
        """Extracts properties from an ARM executable
        Either call this on a sample which is an ARM file. 
        Or from within an embedded ARM file inside an APK.
        """
        if self.properties.filetype == droidutil.ARM or arm_filename != '':
            if arm_filename == '':
                arm_filename = self.absolute_filename
            logging.debug("Extracting properties from " + arm_filename)

            # Run "strings" on the executable
            proc = subprocess.Popen(['strings', arm_filename], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            output = proc.communicate()

            for section in self.properties.armconfig.get_sections():
                matches = re.findall(self.properties.armconfig.get_pattern(section), output[0].decode('utf-8'))
                if matches is not None and len(matches) > 0:
                    self.properties.arm[section] = True
                    logging.debug("Setting arm[%s]  = True" % section)
                    if section == 'url_in_exec':
                        for m in matches:
                            self.interesting_url(m)

    def find_executables(self, dir):
        """Finds executables in a given directory. Does not parse recursively.
        Returns two lists:
        - ARM executables list (absolute filename)
        - APK executables list (absolute filename)
        """
        assert os.path.isdir(dir), "argument should be a directory"
        found_apk = []
        found_arm = []
        listing = os.listdir(dir)
        
        for file in listing:
            absolute_file = os.path.join(dir, file)
            if os.path.isfile(absolute_file):
                filetype = droidutil.get_filetype(absolute_file)

                # Special case to detect Flutter apps compiled in release mode
                if file == 'libapp.so':
                    self.properties.kits['flutter'] = True

                if filetype == droidutil.ARM:
                    logging.debug("%s is an ARM executable" % absolute_file)
                    found_arm.append(absolute_file)
                else:
                    if filetype == droidutil.ZIP:
                        innerzip = droidziprar.droidziprar(absolute_file, True, self.verbose)
                        if innerzip.handle is None:
                            logging.debug("%s is not a valid zip" % absolute_file)
                            return found_apk, found_arm
                        filetype = innerzip.get_type()
                        innerzip.close()
                        if filetype == droidutil.APK:
                            logging.debug("%s contains an APK" % absolute_file)
                            found_apk.append(absolute_file)
                    else:
                        if filetype == droidutil.RAR:
                            innerzip = droidziprar.droidziprar(absolute_file, False, self.verbose)
                            if innerzip.handle is None:
                                logging.debug("%s is not a valid rar" % absolute_file)
                                return found_apk, found_arm
                            filetype = innerzip.get_type()
                            innerzip.close()
                            if filetype == droidutil.APK:
                                logging.debug("%s contains an APK" % absolute_file)
                                found_apk.append(absolute_file)
        return found_arm, found_apk

    def find_exec_in_resources(self):
        """Will detect embedded executables or zips in assets, raw resources and lib dir"""
        asset_dir = os.path.join(self.outdir, "assets")
        raw_dir = os.path.join(self.outdir, "res/raw")
        lib_dir = os.path.join(self.outdir, "lib/armeabi")
        lib2_dir = os.path.join(self.outdir, "lib/arm64-v8a")
        
        list_dir = [asset_dir, raw_dir, lib_dir, lib2_dir]

        for dir in list_dir:
            if os.access(dir, os.R_OK) and os.path.isdir(dir):
                logging.debug("Parsing %s for embedded executables or zips... " % dir)

                found_arm, found_apk = self.find_executables(dir)
                if found_arm or found_apk:
                    self.properties.wide['embed_exec'] = True
                    logging.debug("Embedded executables/zip found in " + dir)
                    if found_arm:
                        for arm in found_arm:
                            logging.debug("Recursively processing " + arm)
                            self.extract_arm_properties(arm)
                    if found_apk:
                        for apk in found_apk:
                            logging.debug("Recursively processing " + apk)
                            droidlysis3.process_file(apk, self.outdir, self.verbose, self.clear, 
                                                     disable_report=self.disable_description,
                                                     no_kit_exception=self.no_kit_exception)

    def interesting_url(self, url):
        """Rules out meaningless URLs to keep only those which might be useful for attackers.
        
        input: url - typically starting with http://
        output: self.properties.url list
        Will set self.properties. urls and some wide properties.
        """
        assert url is not None, "Empty URL is not valid"
        url = re.sub('[,;" ].*', '', url)  # remove after ",; or whitespace
        url = re.sub('\n.*', '', url)
        url = re.sub('\r.*', '', url)
        url = re.sub('[^\x20-\x7e]', '', url)
        
        url_regexp = '|'.join(droidurl.build_special_url_list())
        match = re.search(url_regexp, url)  # only one match per line

        if match is None:
            # the url is not in the list of special URLs
            # let's check it hasn't been reported yet (unique URLs)
            if url not in self.properties.wide['urls']:
                self.properties.wide['urls'].append(url) 
                logging.debug("URL: %s" % url)

            # raise a warning if it downloads APKs or for dropbox
            search = re.findall("\.apk|\.zip", url)
            if search is not None and len(search) > 0:
                if '\.apk' in search or '\.zip' in search:
                    self.properties.wide['apk_zip_url'] = True

        return self.properties.wide['urls']

    def find_base64_strings(self, thedir, exceptions, theproperty):
        """
        To detect base64 strings, we spot all constants with potentially base64 encoded
        characters. Then, on these, we attempt to perform base 64 decoding.
        If that does not lead to an error (padding exception etc) and if the result is
        printable (~ makes potential sense), we assume this is a Base64 string

        const-string v1, "xyz=="
        this won't detect all base64 strings because they won't all finish with ==
        but we'll get some...
        """
        base64_regexp = bytes("const-string v[0-9]*, \"[a-zA-Z0-9/?=]*\"", 'utf-8')
        base64_strings = droidutil.recursive_search(base64_regexp, thedir, exceptions, False)

        if base64_strings is not None:
            printable_chars = bytes(string.printable, 'ascii')
            for s in base64_strings.keys():
                # remove the const-string vX part and trailing "
                thestr = re.sub('const-string v[0-9]*, "', '', s)
                thestr = re.sub('"', '', thestr)
                    
                # try to decode the Base64 string
                try:
                    decoded = base64.b64decode(thestr)
                    
                    if all(c in printable_chars for c in decoded) and decoded != b'' and (not decoded.decode('utf-8') in theproperty):
                        # the decoded string is printable, so likely a valid decoded base64 info
                        # + the array contains unique strings only
                        theproperty.append(decoded.decode('utf-8'))
                        if self.verbose:
                            print("Base64 string: ", decoded)
                except Exception as e:
                    # this is not a base64 string
                    pass

            if len(theproperty) > 0:
                analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
                analysis_file.write("Base64 strings:\n")

                for b in theproperty:
                    analysis_file.write("- "+b+"\n")
                    
                analysis_file.close()

            



import unittest
from droidconfig import generalconfig

'''
To run tests:
source ./venv/bin/activate
python3 setup.py develop
cd tests
python3 ./droidconfig_test.py
'''


class DroidConfigTest(unittest.TestCase):
    def test_general_config(self):
        config = generalconfig(filename='../conf/general.conf', verbose=True)

    def test_bad_config(self):
        self.assertRaises(FileNotFoundError,
                          generalconfig,
                          '../conf/doesnotexist.conf')

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

"""
__author__ = "Axelle Apvrille"
__status__ = "Mature"
__license__ = "MIT License"
"""

def build_special_url_list():
    """Returns a list of special URLs"""
    list = []
    
    # dummy URLs
    list.append('^(http://)127\.0\.0\.1$')
    list.append('^8\.8\.8\.8$')
    list.append('^8\.8\.4\.4$')
    list.append('^https*://%s:%d%s')
    list.append('^(http://)*192\.168\.[0-9.:]*')
    list.append('host:port')
    list.append('^(http://)*localhost$') 
    list.append('^https*://$')
    list.append('^https*$')
    list.append('temporary$') 
    list.append('^https*://unknown$') 
    list.append('^https*://images$')
    list.append('^http://ads$')
    list.append('^http://app$')
    list.append('^http://server$')
    list.append('^http://server/.*')
    list.append('username:password@YOUR')
    list.append('www\.dummyurl\.com')
    list.append('www\.example\.com')
    # clean URLs - which do not correspond to a kit i.e with a smali path
    list.append('creativecommons\.org')
    list.append('docs\.google\.')
    list.append('support\.google\.')
    list.append('developer\.android\.com')
    list.append('developers\.google\.com')
    list.append('googlesyndication\.com')
    list.append('jsoup\.org')
    list.append('www\.jcip\.net')
    list.append('finance\.google\.')
    list.append('maps\.google') 
    list.append('^https*://play\.google\.com')
    list.append('https*://[a-zA-Z]*\.google\.com/') 
    list.append('www\.google\.') 
    list.append('checkout\.google\.com')
    list.append('doubleclick\.net')
    list.append('\.google-analytics\.com') 
    list.append('^https*://[a-z]*\.googleapis\.com/')
    list.append('plus\.url\.google\.com') 
    list.append('market\.android\.com')
    list.append('source\.android\.com')
    list.append('material\.io')
    list.append('java\.sun\.com') 
    list.append('\.facebook\.com/help') 
    list.append('\.facebook\.com$')
    list.append('fonts\.gstatic\.com')
    list.append('forum\.xda-developers\.com/showthread\.php')
    list.append('www\.freetype\.org')
    list.append('www\.microsoft\.com')
    list.append('mozilla\.org') 
    list.append('www\.android\.com') 
    list.append('developer\.android\.com/reference/')
    list.append('fontforge\.sf\.net')
    list.append('www\.apache\.org')  
    list.append('www\.apple\.com')
    list.append('travis-ci\.org')
    list.append('twitter\.com') 
    list.append('www\.gnu\.org')
    list.append('www\.fsf\.org')
    list.append('www\.iec\.ch')
    list.append('www\.paypal\.com')
    list.append('www\.macromedia\.com/go/getflashplayer')
    list.append('www\.mozilla\.org') 
    list.append('opensource\.org') 
    list.append('www\.openssl\.org') 
    list.append('www\.gstatic\.com') 
    list.append('www\.JSON\.org') 
    list.append('www\.junit\.com') 
    list.append('http://www\.amazon\.com/gp/mas/dl/android') 
    list.append('.\.youtube.com') 
    list.append('\.hockeyapp\.net') 
    list.append('http://ns\.adobe\.com/') 
    list.append('https*://[a-zA-Z]*\.jquery\.com')
    list.append('https*://jqueryui\.com')
    list.append('^https*://jquery\.[com|org]/*$')
    list.append('scripts\.sil\.org')
    list.append('www\.ajaxplorer\.info')
    list.append('wikipedia\.org')
    list.append('play\.google\.com')
    list.append('github\.com')
    list.append('jquery\.org')
    list.append('www\.iana\.org')
    list.append('stackoverflow\.com')
    list.append('www\.unicode\.org')
    list.append('freetype\.org')

    # AV
    list.append('www\.fortinet\.com') 
    list.append('docs\.fortinet\.com/fclient/android/')
    list.append('home\.mcafee\.com')
    list.append('\.norton\.com') 
    list.append('www\.avast\.com')
    list.append('\.symantec\.com') 
    list.append('www\.mcafeemobilesecurity\.com/eula\.aspx') 
    list.append('www\.trendmicro\.com') 
    list.append('https*://[a-zA-Z0-9]*\.360safe\.com')

    # search engines
    list.append('search\.twitter\.com') 
    list.append('search\.yahoo\.com') 
    list.append('www\.baidu\.com') 
    list.append('wap\.baidu\.com') 
    list.append('map\.baidu\.com') 
    list.append('www\.searchmobileonline\.com') 


    # XML
    list.append('^https*://push$') 
    list.append('^https*://schemas') 
    list.append('^https*://www\.$') 
    list.append('www\.w3\.org') 
    list.append('xml\.apache\.org') 
    list.append('xml\.org') 
    list.append('xmlpull\.org') 
    list.append('^https*://.*/configure[-_0-9]*\.dtd$')
    
    # Operator
    list.append('10\.0\.0\.172') 
    list.append('10\.0\.0\.200') 
    list.append('wap\.uni-info\.com\.cn') 
    list.append('mmsc\.myuni\.com\.cn') 
    list.append('mmsc\.vnet\.mobi') 
    list.append('wap\.vnet\.mobi') 
    list.append('10\.151\.0\.1') 
    list.append('62\.201\.134\.17') 
    list.append('mmsbouygtel\.com') 
    list.append('\.monternet\.com') 

    # Protocol
    list.append('oauth_token') 

    # adkit urls which sometimes appear outside the smali path
    list.append('pflexads\.com')
    list.append('admob\.com')
    list.append('^https*://api\.weibo\.com')
    list.append('^https*://.*\.alipay.com')

    return list



import unittest
import tempfile
import shutil
from droidlysis3 import process_file
from droidconfig import generalconfig

class DroidLysisTest(unittest.TestCase):
    def setUp(self):
        # Create a temporary directory
        self.test_dir = tempfile.mkdtemp()

    def tearDown(self):
        # Remove the directory after the test
        shutil.rmtree(self.test_dir)

    def test_process_file(self):
        config = generalconfig(filename='../conf/general.conf', verbose=False)
        process_file(config=config,
                     infile='./apk/ph0wn-musicalear.apk',
                     outdir=self.test_dir)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

"""
__author__ = "Axelle Apvrille"
__status__ = "In progress"
__license__ = "MIT License"
"""
import zipfile
import rarfile
import re
import droidutil
import struct
import subprocess
import logging

logging.basicConfig(format='%(levelname)s:%(filename)s:%(message)s', level=logging.INFO)


class droidziprar:

    def __init__(self, archive, zipmode=True, verbose=False):
        """Returns the archive file handle - don't forget to close it once you've finished with the file"""
        if verbose:
            logging.getLogger().setLevel(logging.DEBUG)
        self.zipmode = zipmode  # True for a zip, False for a Rar
        self.password = 'infected'
        self.handle = None
        self.archive_name = archive

        # File type is not fully certain until get_type() has been called
        if self.zipmode:
            self.filetype = droidutil.ZIP
        else:
            self.filetype = droidutil.RAR

        self.open(archive)

    def open(self, filename):
        try:
            if self.zipmode:
                logging.debug("Opening Zip archive "+filename)
                self.handle = zipfile.ZipFile(filename, 'r')
            else:
                logging.debug("Opening Rar archive "+filename)
                self.handle = rarfile.RarFile(filename, 'r')

        except (struct.error, zipfile.BadZipfile, zipfile.LargeZipFile, IOError) as e:
            logging.debug("Exception caught in ZipFile: %s" % (repr(e)))
            self.handle = None

        return self.handle

    def get_type(self):
        """A ZIP file can be really an APK, a JAR, or a real ZIP
        Calling this function reallys sets the appropriate type for the archive.
        Returns:
        - the file type: APK, CLASS or ZIP
        - a list of inner zips/rars if any
        """
        assert self.handle is not None, "zip/rar file handle has been closed"

        innerzips = []  # default value
        files_in_zip = self.handle.namelist()

        # guess file type based on contents
        if [x for x in files_in_zip if re.search("classes\.dex|AndroidManifest\.xml|resources\.arsc|assets/|res/", x)]:
            self.filetype = droidutil.APK
        else:
            innerzips = [x for x in files_in_zip if re.search("\.apk|\.zip|\.rar", x)]
            if innerzips:
                if self.zipmode:
                    self.filetype = droidutil.ZIP
                else:
                    self.filetype = droidutil.RAR
            else:
                if [x for x in files_in_zip if re.search("\.class", x)]:
                    self.filetype = droidutil.CLASS
        return self.filetype, innerzips

    def extract_one_file(self, filename, outdir):
        """Extracts from a ZIP or a RAR
        - file: file to extract
        - outdir: directory to extract to
        Will use a password if necessary
        """
        if self.zipmode:
            # the unzip command works better... (manages to unzip more)
            subprocess.call(["/usr/bin/unzip", "-o", "-qq", "-P",
                             self.password, "-d", outdir, self.archive_name, filename])
        else:
            assert self.handle is not None, "zip/rar file handle has been closed"
            # beware this may raise errors KeyError, RuntimeError...
            self.handle.extract(filename, outdir, pwd=self.password)

    def extract_all(self, outdir):
        if self.zipmode:
            print('Launching unzip process on %s with output dir=%s' % (self.archive_name, outdir))
            subprocess.call(["/usr/bin/unzip", "-o", "-qq", "-P", self.password, self.archive_name, "-d", outdir])
        else:
            assert self.handle is not None, "zip/rar file handle has been closed"
            self.handle.extractall(path=outdir, pwd=self.password)

    def extract_pattern(self, outdir, pattern):
        """Unzips (or unrars) files matching a given regexp to a given output directory.
        Returns a list of what has been unzipped.
        """
        assert self.handle is not None, "zip/rar file handle has been closed"
        all_files = self.handle.namelist()
        list = [x for x in all_files if re.search(pattern, x)]
        
        for file in list:
            self.extract_one_file(file, outdir)

        return list

    def get_date(self, filename):
        """Gets the time at which filename was created"""
        assert self.handle is not None, "zip/rar file handle has been closed"
        try:
            metainfo = self.handle.getinfo(filename)
            return metainfo.date_time
        except (KeyError, rarfile.NoRarEntry) as e:
            logging.debug("%s does not exist in %s" % (filename, self.archive_name))
        return None

    def close(self):
        if self.handle is not None:
            pass
        else:
            self.handle.close()
            logging.debug("Closing archive: " + self.archive_name)
    



#!/usr/bin/env python3

"""
__author__ = "Axelle Apvrille"
__license__ = "MIT License"
"""

import argparse
import requests
import configparser
import json
import re

__version__ = '0.1.0'

def get_arguments():
    parser =  argparse.ArgumentParser(description="Import missing Exodus Trackers in DroidLysis Kits", prog='exodustrack', epilog='Version '+__version__+' - Greetz from Axelle Apvrille')
    parser.add_argument('-i', '--input', help='DroidLysis kit.conf file', action='store', default='./kit.conf')
    parser.add_argument('-v', '--verbose', help='get more detailed messages', action='store_true')
    args = parser.parse_args()
    return args

def get_exodus_trackers(verbose=False):
    r = requests.get('https://etip.exodus-privacy.eu.org/trackers/export',timeout=2)
    assert r.status_code == 200, "Cannot retrieve Exodus Trackers JSON"
    if verbose:
        print("get_exodus_trackers(): status={}".format(r.status_code))
    return r.json()['trackers']

def conf2json(kitfile='./conf/kit.conf', verbose=False):
    parser = configparser.RawConfigParser()
    parser.read(kitfile)
    thelist = []
    for section in parser.sections():
        pattern = parser.get(section, 'pattern')
        try:
            description = parser.get(section, 'description')
        except configparser.NoOptionError as e:
            if verbose:
                print("config2json(): section {} has no description".format(section))
            description = ''
            
        thelist.append( { "section" : section, "pattern" : pattern, "description" : description } )
    return json.loads(json.dumps(thelist))

def compare(exodus, droidlysis, verbose=False):
    # we are going to suggestion addition of any tracker in exodus that is not in droidlysis
    for tracker in exodus:
        if tracker['code_signature'] == '':
            if verbose:
                print("compare(): {} has no code_signature".format(tracker['name']))
        else:
            # convert . to /
            # remove leading and trailing spaces
            # remove trailing /
            # only consider the first pattern when separated by |
            sig = tracker['code_signature'].split('|')[0].replace('.','/').strip().strip('/')
            assert (' ' in sig) == False, "There is a space in sig"
                      
            # search droidlysis patterns
            found = False
            for item in droidlysis:
                for p in item['pattern'].split('|'):
                    if item['pattern'].startswith(sig) or sig.startswith(p):
                        if verbose:
                            print("compare(): {} (sig={}) found in {} (pattern={})".format(tracker['name'],sig, item['section'], p))
                        found = True
                        break

            if not found:
                # this is what you should consider adding
                section_name = ''.join(re.findall('[a-z0-9]',tracker['name'].lower()))
                print("[{}]".format(section_name))
                print("pattern={}".format(sig))
                print("")
            

def main():
    args = get_arguments()
    exodus = get_exodus_trackers(args.verbose)
    droidlysis = conf2json(args.input, args.verbose)
    compare(exodus, droidlysis, args.verbose)
            
if __name__ == "__main__":
    main()
        





#!/usr/bin/env python3

from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='droidlysis',
    description='DroidLysis: pre-analysis of suspicious Android samples',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='@cryptax',
    author_email='aafortinet@gmail.com',
    url='https://github.com/cryptax/droidlysis',
    license='MIT',
    keywords="android malware reverse",
    python_requires='>=3.0',
    version='3.4.7',
    packages=['conf'],
    py_modules=[
        'droidconfig',
        'droidcountry',
        'droidlysis3',
        'droidproperties',
        'droidreport',
        'droidsample',
        'droidsql',
        'droidurl',
        'droidutil',
        'droidziprar',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 3 - Alpha",
        "Operating System :: Unix",
        "Topic :: Software Development :: Disassemblers",
    ],
    include_package_data=True,
    install_requires=[
        'configparser>=4.0.2',
        'python-magic==0.4.12',
        'requests',
        'SQLAlchemy>=1.1.1',
        'rarfile>=3.0',
        'platformdirs'
    ],
    scripts=[ 'droidlysis', 'droidlysis3.py' ]
)



#!/usr/bin/env python3

"""
__author__ = "Axelle Apvrille"
__status__ = "Beta"
__license__ = "MIT License"
"""

import droidutil
import droidconfig
import droidcountry
import droidsql
import json
import requests
import logging
import re
import configparser  # to import Exodus Privacy trackers
from sqlalchemy.orm import sessionmaker
import sqlalchemy.exc

logging.basicConfig(format='%(levelname)s:%(filename)s:%(message)s',
                    level=logging.INFO)


# This is where to set whether given fields have
# a meaning or not for a given file type
applicability = {'file_size': [droidutil.APK, droidutil.DEX, droidutil.ARM,
                               droidutil.CLASS, droidutil.ZIP, droidutil.RAR],
                 'file_small': [droidutil.APK, droidutil.DEX, droidutil.ARM,
                                droidutil.CLASS, droidutil.ZIP, droidutil.RAR],
                 'file_nb_classes': [droidutil.APK, droidutil.DEX],
                 'file_nb_dir': [droidutil.APK, droidutil.DEX],
                 'file_innerzips': [droidutil.APK, droidutil.ZIP, droidutil.RAR],
                 'cert': [droidutil.APK],
                 'manifest': [droidutil.APK],
                 'smali': [droidutil.APK, droidutil.DEX],
                 'wide': [droidutil.APK, droidutil.DEX],
                 'arm': [droidutil.APK, droidutil.ARM],
                 'dex': [droidutil.APK, droidutil.DEX],
                 'kit': [droidutil.APK, droidutil.DEX]
                 }

class droidproperties:
    verbose = False
    """Extracted properties"""

    """Field allocation"""
    certificate = {}
    manifest = {}
    smali = {}
    wide = {}
    arm = {}
    dex = {}
    kits = {}

    def __init__(self, config, samplename='', sha256='', verbose=False, import_exodus=False):
        """Properties concern a given sample identified by a basename (to be helpful) and a sha256 (real reference)"""
        self.config = config
        self.verbose = verbose
        if verbose:
            logging.getLogger().setLevel(logging.DEBUG)
        self.sha256 = sha256
        self.sanitized_basename = samplename
        self.import_exodus = import_exodus
        self.clear_fields()

    def clear_fields(self):
        """Re-initialize all fields of the object - to default values"""
        self.file_nb_classes = 0
        self.file_nb_dir = 0
        self.file_size = 0
        self.file_small = False
        self.filetype = droidutil.UNKNOWN
        self.file_innerzips = False

        self.certificate.clear()
        self.certificate = {'av': False,
                            'algo': None,
                            'debug': False,
                            'dev': False,
                            'famous': False,
                            'serialno': None,
                            'country': droidcountry.country['unknown'],
                            'owner': None,
                            'timestamp': None,
                            'year': 0,
                            'unknown_country': False
                            }

        self.manifest.clear()
        self.manifest = {
            'activities': [],
            'libraries': [],
            'listens_incoming_sms': False,
            'listens_outgoing_call': False,
            'maxSDK': 0,
            'main_activity': None,
            'minSDK': 0,
            'package_name': None,
            'permissions': [],
            'providers': [],
            'receivers': [],
            'services': [],
            'swf': False,
            'targetSDK': 0
        }

        # automatically adding smali properties. 
        self.smali.clear()
        self.smaliconfig = droidconfig.droidconfig(self.config.SMALI_CONFIGFILE, self.verbose)
        for section in self.smaliconfig.get_sections():
            self.smali[section] = False

        self.smali['packed'] = False  # This property is not in conf section as it is deduced from no main activity + loading DEX dynamically
        self.smali['multidex'] = []

        # automatically adding wide properties
        self.wide.clear()
        self.wide['app_name'] = None
        self.wide['phonenumbers'] = []
        self.wide['urls'] = []
        self.wide['base64_strings'] = []
        self.wide['apk_zip_url'] = False
        self.wideconfig = droidconfig.droidconfig(self.config.WIDE_CONFIGFILE, self.verbose)
        for section in self.wideconfig.get_sections():
            self.wide[section] = False

        # automatically add ARM properties
        self.arm.clear()
        self.armconfig = droidconfig.droidconfig(self.config.ARM_CONFIGFILE, self.verbose)
        for section in self.armconfig.get_sections():
            self.arm[section] = False

        self.dex.clear()
        self.dex = {'magic': 0,
                    'odex': False,
                    'magic_unknown': False,
                    'bad_sha1': False,
                    'bad_adler32': False,
                    'big_header': False,
                    'thuxnder': False
                    }

        # automatically set to False kit properties
        self.kits.clear()
        logging.debug(f'Reading Kit Config file = {self.config.KIT_CONFIGFILE}')
        self.kitsconfig = droidconfig.droidconfig(self.config.KIT_CONFIGFILE,
                                                  self.verbose)
        for section in self.kitsconfig.get_sections():
            self.kits[section] = False

        if self.import_exodus:
            self.import_exodus_trackers()
            quit()
        # END OF reinit to default values

    def import_exodus_trackers(self):
        # import ETIP Exodus Privacy trackers
        url = 'https://etip.exodus-privacy.eu.org/api/trackers/?format=json'
        logging.debug(f'Importing ETIP Exodus trackers from {url}')
        r = requests.get(url)
        if r.status_code != 200:
            logging.warning('Cannot download Exodus Privacy trackers: '
                            f'{url} responds code={r.status_code}')
            return
        j = json.loads(r.text)
        config = configparser.ConfigParser()
        for tracker in j:
            # remove trailing dots
            code_signature = re.sub('\.\|', '|', tracker['code_signature']).rstrip('.').lstrip('.').replace('.', '/').replace('\/', '/').split('|')
            for sig in code_signature:
                if len(re.sub('[^a-zA-Z0-9/_]', '', sig)) == 0:
                    continue
                if self.kitsconfig.is_pattern_present(sig):
                    # logging.debug(f'Not adding tracker={tracker["name"]}'
                    #              f' as pattern={sig} is already present')
                    break
                # sanitize name
                name = re.sub('[^a-zA-Z0-9]', '', tracker['name']).lower()
                # if our signature is more generic, nothing to do
                if name in self.kits:
                    # our signature is less generic / missing a pattern
                    logging.debug(f'name={name} sig={sig} pattern={self.kitsconfig.get_pattern(name)}')
                    logging.warning(f'You should add pattern={sig}'
                                    f' in tracker={tracker["name"]}')
                    break
                logging.debug(f'Adding Exodus Tracker: {name}')
                config[name] = {}
                config[name]['description'] = f'{tracker["name"]} (from ETIP Exodus Privacy list)'
                config[name]['pattern'] = '|'.join(code_signature)
                break
        logging.debug('Appending imported trackers '
                      f'to {self.config.KIT_CONFIGFILE}')
        with open(self.config.KIT_CONFIGFILE, 'a') as configfile:
            config.write(configfile)

    def write(self, sql):
        Session = sessionmaker(bind=sql.engine)
        session = Session()
        sample = droidsql.Sample(sha256=self.sha256,
                                 sanitized_basename=self.sanitized_basename,
                                 file_nb_classes=self.file_nb_classes,
                                 file_nb_dir=self.file_nb_dir,
                                 file_size=self.file_size,
                                 file_small=self.file_small,
                                 filetype=self.filetype,
                                 file_innerzips=self.file_innerzips,
                                 manifest_properties=json.dumps(self.manifest),
                                 smali_properties=json.dumps(self.smali),
                                 wide_properties=json.dumps(self.wide),
                                 arm_properties=json.dumps(self.arm),
                                 dex_properties=json.dumps(self.dex),
                                 kits=json.dumps(self.kits))
        session.add(sample)
        try:
            session.commit()
        except sqlalchemy.exc.IntegrityError:
            # occurs when the sample with the same sha256 is already in
            logging.debug("Sample is already in the database")

    def dump_json(self, filename='report.json'):
        data = {'sanitized_basename': self.sanitized_basename,
                'file_nb_classes': self.file_nb_classes,
                'file_nb_dir': self.file_nb_dir,
                'file_size': self.file_size,
                'file_small': self.file_small,
                'filetype': self.filetype,
                'file_innerzips': self.file_innerzips,
                'manifest_properties': self.manifest,
                'smali_properties': self.smali,
                'wide_properties': self.wide,
                'arm_properties': self.arm,
                'dex_properties': self.dex,
                'kits': self.kits
                }
        if self.verbose:
            print("-------------")
            print("Dumping to JSON file {}".format(filename))
        f = open(filename, 'w')
        f.write(json.dumps(data))
        f.close()
